import typing
import os


def detect_tasks(root_dirpath: str) -> tuple:
    """
    Detect Celery tasks from a root directory
    """
    tasks = []
    file_path = os.path.join(root_dirpath)
    for root, dirs, files in os.walk(file_path):
        for filename in files:
            if os.path.basename(root) == "tasks":
                if filename != "__init__.py" and filename.endswith(".py"):
                    task = (
                        os.path.join(root, filename)
                        .replace(root_dirpath + "/", "")
                        .replace("/", ".")
                        .replace(".py", "")
                    )
                    tasks.append(task)
    return tuple(tasks)


def detect_periodic_tasks(root_dirpath: str) -> typing.Dict:
    """
    Detect Celery periodic tasks from a root directory
    """
    tasks = {}
    file_path = os.path.join(root_dirpath)
    for root, dirs, files in os.walk(file_path):
        for filename in files:
            if os.path.basename(root) == "tasks":
                if filename.startswith("periodic_tasks"):
                    full_path = os.path.join(root, filename)
                    ldict = locals()
                    exec(open(full_path).read(), globals(), ldict)
                    try:
                        tasks.update(ldict["periodic_tasks"])
                    except NameError:
                        pass
    return tasks


class RetriableTaskException(Exception):
    """
    Exception for celery tasks that indicates task can be retried
    """

    pass


class UnRetriableTaskException(Exception):
    """
    Exception for celery tasks that indicates task cannot be retried
    """

    pass



import json

from django.http import HttpResponse
from rest_framework import status
from rest_framework.views import exception_handler


def error_404_view(request, exception=None):
    response = {"status_code": 404, "error": "The resource was not found"}
    return HttpResponse(
        json.dumps(response),
        content_type="application/json",
        status=status.HTTP_404_NOT_FOUND,
    )


def error_500_view(request, exception=None):
    response = {"status_code": 500, "error": "Internal server error"}
    return HttpResponse(
        json.dumps(response),
        content_type="application/json",
        status=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


def custom_exception_handler(exc, context):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    response = exception_handler(exc, context)

    if response is not None:
        # check if exception has dict items
        if hasattr(exc.detail, "items"):
            # remove the initial value
            response.data = {}
            errors = []
            for key, value in exc.detail.items():
                # append errors into the list
                errors.append("{} : {}".format(key, " ".join(value)))

            # add property errors to the response
            response.data["errors"] = errors

        # serve status code in the response
        response.data["status_code"] = response.status_code

    return response



import datetime
import decimal
import json
import typing
import uuid
from json import JSONEncoder

import numpy
import pandas as pd


class CustomJSONEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()

        if isinstance(obj, datetime.date):
            return obj.isoformat()

        if isinstance(obj, decimal.Decimal):
            return float(obj)

        if isinstance(obj, pd.Interval):
            if obj.left == float("-inf"):
                return f"-{obj.right}"
            if obj.right == float("inf"):
                return f"{obj.left}+"
            return f"{obj.left}-{obj.right}"

        if isinstance(obj, uuid.UUID):
            return str(obj)

        if isinstance(obj, pd.Timestamp):
            return obj.isoformat()

        if isinstance(obj, numpy.generic):
            return numpy.asscalar(obj)

        if isinstance(obj, numpy.ndarray):
            return obj.to_list()

        if isinstance(obj, float) and numpy.isnan(obj):
            return "null"

        if isinstance(obj, numpy.integer):
            return int(obj)

        elif isinstance(obj, numpy.floating):
            return float(obj)

        return super().default(obj)


def remap_keys(mapping, value_name="value"):
    return [{"group": k, value_name: v} for k, v in mapping.items()]


def remap_timeseries(mapping, value_name="value"):
    return [{"date": k, value_name: v} for k, v in mapping.items()]


def convert_numpy_objects(dict_to_convert: typing.Dict) -> typing.Dict:
    new = {}
    for k, v in dict_to_convert.items():
        if isinstance(v, dict):
            new[k] = convert_numpy_objects(v)
        else:
            if isinstance(v, float) and (numpy.isnan(v) or numpy.isinf(v)):
                new[k] = None
            else:
                new[k] = v
    return new


def json_dumps(rmd):
    if isinstance(rmd, dict):
        rmd_new = convert_numpy_objects(rmd)
    elif isinstance(rmd, list):
        rmd_new = [convert_numpy_objects(item) for item in rmd]
    else:
        rmd_new = rmd
    return json.dumps(rmd_new, cls=CustomJSONEncoder, allow_nan=False)


def json_loads(rmd):
    return json.loads(json_dumps(rmd))
